#!/usr/bin/perl

use strict;
use Getopt::Std ;
use vars qw($opt_m $opt_t $opt_g $opt_d $opt_v $opt_h );

#declaration des variables
my $version;
my $date;
my $email;
my $nom_liste;
my $path;
my $domaine;
my $file;
my $buffer;
my $file_config_tpl;
my $type;
my $groupe;

$version = 2.1;
my $copyright = "\n";
$copyright .= "    create-list.pl v. $version.\n\n";
$copyright .= "    Utilitaire de création d'un liste Sympa .\n";
$copyright .= "    Copyright (c) 2003 Joel Cuissinat <joel.cuissinat\@ac-dijon.fr>.\n\n";
$copyright .= "    Copyright (c) 2002 Kévin Reverchon <kevin.reverchon\@ac-dijon.fr>.\n\n";
$copyright .= "    create-list.pl EST UN LOGICIEL LIBRE. SE RÉFÉRER AU CODE SOURCE\n";
$copyright .= "    POUR LA LICENCE D'UTILISATION ET DE COPIE.\n\n";

my $help = "syntaxe pour utiliser create-list.pl\n";
$help .= "./create-list.pl
-g nom du groupe associé à la liste
-d domaine de la liste (etab.ac-aca.fr ou i-etab.ac-aca.fr)
-t type de liste (Classe, Niveau, Equipe,...)
-m le mail de l'administrateur de la liste\n";
$help .= "./create-list.pl -v: affiche le copyright\n";
$help .= "./create-list.pl -h: affiche ce message\n";

unless( getopts('m:t:g:d:vh') )
{
    #print STDERR $help;
    print "Erreur dans les options !";
    exit 0;
}

#affichage la version et le copyright si option -v
if( $opt_v )
{
   print STDERR $copyright;
    exit 0;
}

#affichage de l'aide si option -h
if($opt_h) 
{
    print STDERR $help;
    exit 0;
}


# groupe <=> nom de la liste 
if($opt_g) {
   #visiblement c un filtre a caractères spéciaux ?
   $_ = $opt_g;
   tr/A-Z/a-z/;
   $groupe = $_; 
} else {
   print "Erreur option g";
   exit 0;
}

# type de liste
if($opt_t) {
   $type = $opt_t;
} 
else {
   $type = "Autre";
}

# domaine
if($opt_d) 
{
  $domaine = $opt_d;
} else {
   print "Erreur option d";
   exit 0;
}

# mail administrateur
if($opt_m) 
{
   $email = $opt_m;
} else {
   print "Erreur option m";
   exit 0;
}


#initialisation des variables path
$file_config_tpl = "/usr/share/ead/liste-sympa.tpl";
if ($domaine =~ m/^i-/){
   # domaine interne
   $path = "/var/lib/sympa/expl/$domaine/";
} else {
   # domaine externe
   $path = "/var/lib/sympa/expl/";
}

#nom de la liste de diffusion a creer
$nom_liste = $groupe;

if (-e "$path$nom_liste") {
    print "La liste de diffusion $path$nom_liste existe deja\n";
    exit 0;
}

#creation du repertoire de la liste
`mkdir $path$nom_liste`;

#creation du fichier info
`touch $path$nom_liste/info`;

# visiblement, il vaut mieux laisser SYMPA gérer ces trucs là
#creation du repertoire archive de la liste
#`mkdir $path$nom_liste/archives`;
#creation des fichier subscibers
#`touch $path$nom_liste/subscribers.db`;
#creation du fichier stats
#`touch $path$nom_liste/stats`;
#creation du fichier msg_count
#`touch $path$nom_liste/msg_count`;


#######################################################################################################
#
#                                           PARTIE TEMPLATE
#
#
#####################################################################################################

#initialisation de la date
$date = localtime();
$file = "$path$nom_liste/config";
my %data = ( 
	     'email' => $email
	     ,'date' => $date
	     ,'groupe' => $groupe
	     ,'type' => $type
	     ) ;

# création du fichier de config de la liste

use Template;
my $tt = Template->new (( ABSOLUTE=>1 )) ;

$tt->process($file_config_tpl, \%data, $file)
        || die $tt->error;
#print "Votre liste a été crée";



#######################################################################################################
#                                               PARTIE ETC/ALIASES
#
######################################################################################################
open(ALIAS, ">>/etc/mail/sympa_aliases");

$buffer  =  "$nom_liste: \"|/usr/lib/sympa/bin/queue  $nom_liste\@$domaine\"\n";
$buffer .=  "$nom_liste-request: \"|/usr/lib/sympa/bin/queue $nom_liste-request\@$domaine\"\n";
$buffer .=  "$nom_liste-editor: \"|/usr/lib/sympa/bin/queue $nom_liste-editor\@$domaine\"\n";
$buffer .=  "$nom_liste-owner: \"|/usr/lib/sympa/bin/bouncequeue $nom_liste\@$domaine\"\n";
#$buffer .=  "$nom_liste-unsubscribe: \"|/usr/lib/sympa/bin/queue $nom_liste-unsubscribe\@$domaine\"\n\n";

print ALIAS $buffer;

close(ALIAS);

`chown -R sympa.sympa /var/lib/sympa`;
