#!/usr/bin/env python
# -*- coding: UTF-8 -*-

#    Conception :
#    Eole (http://eole.orion.education.fr)

#    Ce code a été développé par
#    Klaas TJEBBES
#    Joël CUISSINAT

#    distribué sous la licence GPL-2

from genmodelesinc import *
from genlecteurmask import *
from xml.dom.minidom import parse
from copy import copy
from os import system
import sys

def unix_v_dos(fichier):
    """
    Conversion d'un fichier texte au format DOS
    """
    cmd = '/usr/bin/todos -u ' + fichier
    system(cmd)

def ouvrirFich(fich,modelect):
    """
    Ouverture d'un fichier avec test d'existance
    """
    try:
        monfich = file(fich,modelect)
    except:
        print 'Fichier '+fich+' introuvable'
        sys.exit(1)
    return monfich

def listerRegles(documentXml):
    """
    Renvoit la liste des regles contenues dans registre.xml
    """
    documentXml = parse(documentXml)
    listeRegles = documentXml.getElementsByTagName("Regle")
    return listeRegles

def genDicoEad(fichModele):
    """
    Crée un dico à partir des données de l'EAD
    """
    dicoEad = {}
    fichModele = ouvrirFich('/var/www/ead/modeles/groupes/'+fichModele,'r')
    try:
        for ligne in fichModele:
            mavar = ligne.strip().split('#')
            dicoEad[mavar[0]] = mavar[1]
    except:
        pass
    fichModele.close()
    return dicoEad

def genListeDicoReg(listeRegles):
    listeDicoReg = []
    listeDicoRegXP = []
    listeOs = ['224','252']
    listeOsXP = ['4','12','252']
    for i in listeRegles:
        # Attention : si on fait l.append(dico), 
        # les 2 listes pointent sur le même objet python !
        # => faire une copie
        dico = extraitValeur(i)
        if dico['os'] in listeOs:
            listeDicoReg.append(copy(dico))
        if dico['os'] in listeOsXP:
            listeDicoRegXP.append(copy(dico))
    return listeDicoReg, listeDicoRegXP

def genListeDicoRegFinal(listeRegles, fichModele):
    """
    Génération de deux listes de dictionnaires 
    en fonction de l'OS
    """
    dicoEad = genDicoEad(fichModele)
    listeDicoReg, listeDicoRegXP = genListeDicoReg(listeRegles)
    #9x ...
    for i in listeDicoReg:
        # valeur utilisateur
        if dicoEad.has_key(i['id']) :
            valeur = dicoEad[i['id']]
            if valeur != '1':
                i['valeur'] = valeur
        # DWORD non défini
        elif i['type_variable'] == 'DWORD':
            i['valeur'] = '0'
        # CHAINE non définie
        elif i['type_variable'] == 'CHAINE':
            i['valeur'] = i['valeur_decoche']
        # type_variable inconnue
        else :
            pass
        i['valeur'] = i['valeur'].replace('\\','\\\\')
        # lettres de lecteur pour 9x
        if i['id'] == '301':
            i['valeur'] = hex(genMask(i['valeur'])).split('x')[1]
    listeDicoRegFinal = listeDicoReg
    #XP ...
    for j in listeDicoRegXP:
        # valeur utilisateur
        if dicoEad.has_key(j['id']) :
            valeur = dicoEad[j['id']]
            if valeur == '0':
                j['valeur'] = j['valeur_decoche']
            elif valeur != '1':
                j['valeur'] = valeur
        # DWORD non défini
        elif j['type_variable'] == 'DWORD':
            j['valeur'] = '0'
        # CHAINE non définie
        elif j['type_variable'] == 'CHAINE':
            j['valeur'] = j['valeur_decoche']
        # type_variable inconnue
        else :
            pass
        # lettres de lecteur pour XP
        if j['id'] == '300':
            j['valeur'] = genMask(j['valeur'])
    listeDicoRegFinalXP = listeDicoRegXP
    return listeDicoRegFinal, listeDicoRegFinalXP

def ecriReg(listeRegFinal,modele):
    #pour 98 uniquement
    liste = ['REGEDIT4\n\n']
    montemplate = """[HKEY_CURRENT_USER\%(chemin)s]\n"%(variable)s"=%(type_variable)s%(valeur)s\n"""
    for i in listeRegFinal:
        if i['type_variable'] == 'DWORD':
            i['type_variable'] = 'dword:'
        if i['type_variable'] == 'CHAINE':
            i['type_variable'] = ''
            i['valeur'] = '"'+i['valeur']+'"'
        liste.append(montemplate % i)
    regwin = ouvrirFich('/home/netlogon/scripts/os/Win95/'+modele+'.reg','w')
    for i in liste:
        regwin.write(i)
    regwin.write('\n')
    regwin.close()
    unix_v_dos('/home/netlogon/scripts/os/Win95/'+modele+'.reg')

def ecritXP(listeRegFinalXP,modele):
    regwin = ouvrirFich('/home/netlogon/scripts/os/WinXP/'+modele,'w')
    regwin.write('regliste='+str(listeRegFinalXP))
    regwin.close()
    from shutil import copyfile
    copyfile('/home/netlogon/scripts/os/WinXP/'+modele,'/home/netlogon/scripts/os/Win2K/'+modele)

def main(fichXml, fichModele):
    # génération des dictionnaires de règles
    listeDef, listeDefXP = genListeDicoRegFinal(listerRegles(fichXml),fichModele)
    # écriture à plat pour Win9x :
    ecriReg(listeDef, fichModele)
    # écriture sous forme de dictionnaire pour XP et 2000 :
    ecritXP(listeDefXP, fichModele)

if __name__ == "__main__":
    try:
        fichXml = '/var/www/ead/modeles/registre.xml'
        fichModele=sys.argv[1]
        main(fichXml,fichModele,)
    except:
        print "erreur"

