#!/usr/bin/perl
use strict;
use Getopt::Std ;
use vars qw($opt_l $opt_p $opt_o $opt_b $opt_v $opt_h $opt_d $opt_c );

use Net::LDAP qw(:all);                      # use for all code
use Net::LDAP::Util qw(ldap_error_name       
                          ldap_error_text) ;    # use for Error handling

my $domain_mail;
my $ldap;
my $file_ldif;
my $mesg;
my $recup;
my @recup_mail;
my $classe;
my $login;
my $password;
my $version;
my $branche;


#######################################################################################################                                          gestion des options
######################################################################################################
$version = 2;
my $copyright = "\n";
$copyright .= "    modify-mail.pl v. $version.\n\n";
$copyright .= "    Utilitaire d'index d'un fichier de log.\n";
$copyright .= "    Copyright (c) 2003 Joel Cuissinat <joel.cuissinat\@ac-dijon.fr>.\n\n";
$copyright .= "    Copyright (c) 2002 Kévin Reverchon <kreverchon\@ac-dijon.fr>.\n\n";
$copyright .= "    modify-mail.pl EST UN LOGICIEL LIBRE. SE RÉFÉRER AU CODE SOURCE\n";
$copyright .= "    POUR LA LICENCE D'UTILISATION ET DE COPIE.\n\n";

my $help = "syntaxe pour utiliser modify-mail.pl\n";
$help .= "./modify-mail.pl -l login de l'administrateur -p le password -o fichier de sortie -b bom de la branche dans l'annuaire -d domain mail -c la classe \n";
$help .= "./modify-mail.pl -v: affiche le copyright\n";
$help .= "./modify-mail.pl -h: affiche ce message\n";
      
unless( getopts('l:p:o:b:d:c:vh') )
{
   print STDERR $help;
    exit 0;
}

#affichage la version et le copyright si option -v

if( $opt_v )
{
   print STDERR $copyright;
    exit 0;
}

#affichage de l'aide si option -h

if($opt_h) 
{
    print STDERR $help;
    exit 0;
}

#gestion de l'option l (login de l'administrateur de ldap)
# Si l'option -l manque, affichage de l'aide

if( $opt_l ) 
{
    $login = $opt_l ;
} else {
    print STDERR $help;
    exit 0;
}


# gestion de l'option p (password de l'administrateur de ldap)
# si l'loption -p manque, affichage de l'aide
if ( $opt_p )
{
    $password = $opt_p ;
} else {
    print STDERR $help ;
    exit 0 ;
}

#gestion de l'option o (nom du fichier de sortie)
# si l'option -o manque, affichage de l'aide
if ( $opt_o )
{
    $file_ldif = $opt_o ;
} else {
    print STDERR $help ;
    exit 0 ;
}

#gestion de l'option b (nom de la branche)
# si l'option -b manque, affichage de l'aide
if ( $opt_b )
{
    $branche = $opt_b ;
} else {
    print STDERR $help ;
    exit 0 ;
}

#gestion de l'option d (nom du domaine mail)
# si l'option -d manque, affichage de l'aide
if ( $opt_d )
{
    $domain_mail = $opt_d ;
} else {
    print STDERR $help ;
    exit 0 ;
}


#gestion de l'option c (classe a chercher)
# si l'option -c manque, affichage de l'aide
if ( $opt_c )
{
    $classe = $opt_c ;
} else {
    print STDERR $help ;
    exit 0 ;

}

#gestion de l'option s (section a chercher)
# si l'option -s manque, affichage de l'aide
#if ( $opt_s ){ $section = $opt_s ;} else {    print STDERR $help ;  exit 0 ;}

#######################################################################################################                                        fin de la gestion des options
######################################################################################################
			  
       

#######################################################################################################
#                                   programme principal
#
######################################################################################################

$ldap = Net::LDAP->new("localhost") or die "$@";
$mesg = $ldap->bind($login,           
                        password => $password,
                        version => 3 );

my @Attrs = ();             # request all available attributes
                                # to be returned.
my $result = LDAPsearch($ldap,"(Divcod=$classe)",\@Attrs);

my @entries = $result->entries;
my $cpt=0;

my $entr ;
open (FILE_MODIF, ">$file_ldif") ;
        foreach $entr ( @entries )
           {
	#print FILE_MODIF "\n\n";
my $attr;
           foreach $attr ( sort $entr->attributes ){

                next if ( $attr =~ /;binary$/ );
		$recup = $entr->get_value($attr);

		@recup_mail = split(/@/, $recup);

                }
		$cpt++;
#ecriture des donnnees a modifier dans un fichier
	    print FILE_MODIF "dn: ".$entr->dn."\n";
	    print FILE_MODIF "changetype: modify\n";
	    print FILE_MODIF "replace: mail\n";
	    print FILE_MODIF "mail:".$recup_mail[0]."@".$domain_mail."\n\n";
           }
print $cpt;

close (FILE_MODIF);




`ldapmodify -x -v -D "cn=admin,o=gouv,c=fr" -w $password -f $file_ldif`;



#######################################################################################################                                    fonction ldapsearch: debut
######################################################################################################


 sub LDAPsearch
        {
          my ($ldap,$searchString,$attrs,$base) = @_ ;


 $base = $branche; 



$attrs = ['mail'] ;

my $result = $ldap->search (
                base    => "$base",
                scope   => "sub",
                filter  => "$searchString",
                attrs   => $attrs
                );



}

#######################################################################################################                                         fin fonction ldapsearch
######################################################################################################
