#!/usr/bin/env python
# -*- coding: UTF-8 -*-
"""
Parseur DOM du fichier registre.xml pour l'affichage des options
"""

from xml.dom.minidom import parse
import string

def parse_fichier(fichier):
    """parseur du fichier registre.xml
    retourne un dictionnaire contenant la liste des 'Rubriques' que contiennent chaque 'Categories'
    """
    doc = parse(fichier)
    cats = doc.getElementsByTagName('Categorie')
    dico_cat = {}
    for cat in cats:
        libelle, res = parse_categorie(cat)
        dico_cat[libelle] =res
    return dico_cat

def parse_categorie(cat):
    """parseur de la liste d'éléments DOM 'Categorie'
    """
    libel_cat = get_text(get_first_elt(cat, 'Nom'))
    dico_rub = {}
    for rub in cat.getElementsByTagName('Rubrique'):
        libel_rub, res = parse_rubrique(rub)
        dico_rub[libel_rub] = res
    #certaines catégorie n'ont pas de rubriques mais directement des règles
    regle_orph = [e for e in cat.childNodes if e.nodeName == 'Regle']
    liste_reg_orph = []
    if len(regle_orph) != 0:
	for regle in regle_orph:
            liste_reg_orph.append(parse_regles(regle))
        dico_rub['AAAA'] = liste_reg_orph
    return libel_cat, dico_rub

def parse_rubrique(rub):
    """parseur d'une rubrique
    """
    libel_rub = get_text(get_first_elt(rub, 'Nom'))
    liste_reg = []
    for regle in rub.getElementsByTagName('Regle'):
        liste_reg.append(parse_regles(regle))
    return libel_rub, liste_reg
    
def parse_regles(regle):
    """parseur de regles
    récupère ID, OS, Intitule, Variable, Composant
    """
    dico_reg = {}
    dico_reg['ID'] = get_text(get_first_elt(regle, 'ID'))
    dico_reg['OS'] = get_text(get_first_elt(regle, 'OS'))
    dico_reg['Intitule'] = get_text(get_first_elt(regle, 'Intitule'))
    dico_reg['Variable'] = get_text(get_first_elt(regle, 'Variable'))
    dico_reg['Composant'] = get_text(get_first_elt(regle, 'Composant'))
    try:
        dico_reg['ValeurSiDecoche'] = get_text(get_first_elt(regle, 'ValeurSiDecoche')).replace('\\','\\\\')
    except:
        pass
    return dico_reg

def get_text(node):
    """
    Renvoie la valeur textuelle d'un node simple (c'est-à-dire ne contenant que du texte)
    """
    text_node = node.childNodes[0]
    return text_node.data.encode('iso-8859-1').replace("'","\\'")

def get_first_elt(node, tag):
    """
    Renvoie la première balise tag présente dans node
    """
    return node.getElementsByTagName(tag)[0]

def dico_to_array(dico):
    if type(dico) == list:
        return 'Array(' + string.join(
                  map( lambda i: dico_to_array(i), dico),
                ", " ) + ")"

                
    if type(dico) == dict:
        return "Array(" + string.join( 
                map( lambda k,d=dico: dico_to_array(k)+" => "+dico_to_array(d[k]), 
                dico.keys() ), ", " ) + ")"
    if type(dico) == str:
	return "'"+dico+"'"
    return dico

if __name__ == "__main__":
    print dico_to_array(parse_fichier('/var/www/ead/modeles/registre.xml'))
