#!/bin/bash

#############################################
# EOLE SCRIBE
# Mise en place d'un répertoire classe
# 04/12/2003
# Joël Cuissinat <joel.cuissinat.ac-dijon.fr>
#############################################

# utilisation :
# sh rep-classe.sh classe

# Prérequis :
# La classe et son équipe pédagogique doivent déjà exister

# Résultats :
#	/home/classe/$classe (contiendra les liens vers les élèves)
#	/home/workgroups/$classe/- donnees (élèves en lecture seule)
#				 - travail (élèves en lecture/ecriture)
#	/home/workgroups/profs-$classe/- eleves (lien vers les élèves)
#				       - classe (lien sur le réperoire classe)

# récupération de l'argument classe
CLASSE=$1

# chemins
GROUPES="home/workgroups"

# création du répertoire où seront stockés les liens sur les élèves de la classe
mkdir -p /home/classes/$CLASSE
# droits d'accès sur le réperoire contenant les liens élèves
setfacl -P -m u::rwx,g::---,o::--- /home/classes/$CLASSE
setfacl -P -m g:professeurs:r-x /home/classes/$CLASSE

# -- Arborescence répertoire classe ($classe) --

# répertoire de base
mkdir -p /$GROUPES/$CLASSE
# droit unix
setfacl -PRm u::rwx,g::---,o::--- /$GROUPES/$CLASSE
# droits d'accès pour tous les professeurs sur la classe
setfacl -PRm g:professeurs:r-x /$GROUPES/$CLASSE
# droits d'accès pour tous les élèves
setfacl -PRm g:${CLASSE}:r-x /$GROUPES/$CLASSE

# répertoire en RW
mkdir -p /$GROUPES/$CLASSE/travail
setfacl -PRm g:professeurs:rwx /$GROUPES/$CLASSE/travail
setfacl -PRm g:${CLASSE}:rwx /$GROUPES/$CLASSE/travail

# répertoire en RO
mkdir -p /$GROUPES/$CLASSE/donnees
setfacl -PRm g:professeurs:rwx /$GROUPES/$CLASSE/donnees
setfacl -Pm g:${CLASSE}:r-x /$GROUPES/$CLASSE/donnees

#Les acls par défaut
getfacl --access --absolute-names /$GROUPES/$CLASSE | setfacl -d -M- /$GROUPES/$CLASSE
getfacl --access --absolute-names /$GROUPES/$CLASSE/donnees | setfacl -d -M- /$GROUPES/$CLASSE/donnees
getfacl --access --absolute-names /$GROUPES/$CLASSE/travail | setfacl -d -M- /$GROUPES/$CLASSE/travail

# -- Arborescence équipe pédagogique (profs-$classe) --
# lien vers la classe dans l'équipe
ln -nsf /$GROUPES/$CLASSE /$GROUPES/profs-$CLASSE/classe
# lien vers le répertoire contenant les élèves
ln -nsf /home/classes/$CLASSE /$GROUPES/profs-$CLASSE/eleves

