<?php

include_once ('TafelTreeBranch.class.php');

/**
 * 
 *
 * @author 	FTafel
 */
class TafelTree {
	
	
	/**
	 *------------------------------------------------------------------------------
	 *							Propriétés
	 *------------------------------------------------------------------------------
	 */
	
	/**
	 * @access 	protected
	 * @var 	string			$id						L'id de l'arbre
	 */
	var $id;
	var $multiline;
	var $checkboxes;
	var $checkboxesThreeState;
	var $cookies;
	var $cookieSeparator;
	var $openOnlyOne;
	var $openedAfterAdd;
	var $openedAtLoad;
	var $selectedBranchShowed;
	var $propagate;
	var $lineStyle;
	var $bigTree;
	var $branches;

	var $onLoad;
	
	
	
	/**
	 *------------------------------------------------------------------------------
	 *							Constructeur
	 *------------------------------------------------------------------------------
	 */
	
	/**
	 * Constructeur
	 *
	 * @access	public
	 * @param	string				id						L'id de l'élément HTML conteneur
	 * @param	string				imgBase					Le path vers les images
	 * @param	integer				width					La largeur de l'arbre
	 * @param	integer				height					La hauteur de l'arbre
	 */
	function TafelTree ($id, $imgs = 'imgs/', $width = '100%', $height = 'auto'){
		$this->id = $id;
		$this->imgs = $imgs;
		$this->width = $width;
		$this->height = $height;
		
		$this->multiline = 0;
		$this->checkboxes = 0;
		$this->checkboxesThreeState = 0;
		$this->cookies = 0;
		$this->cookieSeparator = 0;
		$this->openOnlyOne = 0;
		$this->openedAfterAdd = 1;
		$this->openedAtLoad = 0;
		$this->selectedBranchShowed = 1;
		$this->propagate = 1;
		$this->bigTree = 0;
		$this->setLineStyle('line');
		$this->branches = array();
	}
	
	
	
	/**
	 *------------------------------------------------------------------------------
	 *							Fonctions getters et setters
	 *------------------------------------------------------------------------------
	 */
	
	function getId() {
		return $this->id;
	}
	function getImgs() {
		return $this->imgs;
	}
	function getWidth() {
		return $this->width;
	}
	function getHeight() {
		return $this->height;
	}
	
	function enableMultiline ($enable) {
		$this->multiline = ($enable) ? 1 : 0;
	}
	function getMultiline () {
		return $this->multiline;
	}
	
	function enableCheckboxes ($enable) {
		$this->checkboxes = ($enable) ? 1 : 0;
	}
	function getCheckboxes () {
		return $this->checkboxes;
	}
	
	function enableCheckboxesThreeState ($enable) {
		$this->checkboxesThreeState = ($enable) ? 1 : 0;
	}
	function getCheckboxesThreeState () {
		return $this->checkboxesThreeState;
	}
	
	function enableCookies ($enable, $separator = null) {
		$this->cookies = ($enable) ? 1 : 0;
		if ($separator) {
			$this->cookieSeparator = $separator;
		}
	}
	function getCookies () {
		return $this->cookies;
	}
	function getCookiesSeparator () {
		return $this->cookieSeparator;
	}
	
	function openOneAtOnce ($enable) {
		$this->openOnlyOne = ($enable) ? 1 : 0;
	}
	function getOpenAtOnce () {
		return $this->openOnlyOne;
	}
	
	function openAfterAdd ($enable) {
		$this->openedAfterAdd = ($enable) ? 1 : 0;
	}
	function getOpenAfterAdd () {
		return $this->openedAfterAdd;
	}
	
	function openAtLoad ($enable) {
		$this->openedAtLoad = ($enable) ? 1 : 0;
	}
	function getOpenedAtLoad () {
		return $this->openedAtLoad;
	}
	
	function showSelectedBranch ($enable) {
		$this->selectedBranchShowed = ($enable) ? 1 : 0;
	}
	function getShowSelectedBranch () {
		return $this->selectedBranchShowed;
	}
	function loadForBigTree($load) {
		$this->bigTree = ($load) ? 1 : 0;
	}
	
	function setLineStyle ($style) {
		switch ($style) {
			case 'none':
			case 'line': $this->lineStyle = $style;
			default:
				$this->lineStyle = 'line';
		}
	}
	function getLineStyle () {
		return $this->lineStyle;
	}
	
	function propagateRestriction ($enable) {
		$this->propagate = ($enable) ? 1 : 0;
	}
	function getPropagateRestriction () {
		return $this->propagate;
	}
	
	
	
	/**
	 *------------------------------------------------------------------------------
	 *							Fonctions de fonctions javascript
	 *------------------------------------------------------------------------------
	 */
	function setOnLoad ($func) {
		$this->onLoad = $func;
	}
	
	
	/**
	 *------------------------------------------------------------------------------
	 *							Fonctions publiques
	 *------------------------------------------------------------------------------
	 */
	
	function addBranch ($id, $txt) {
		$branch = new TafelTreeBranch ($id, $txt);
		$this->branches[] = $branch;
		return $branch;
	}
	
	function init () {
		$strp = '';
		if (isset($this->onLoad)) {
			$strp .= "
				tree.setOnLoad(".$this->onLoad.");
			";
		}
		$str = "
			var tree = null;
			function TafelTreeInit() {
				var struct = ".$this->getStructure().";
				tree = new TafelTree ('".$this->getId()."', struct, '".$this->getImgs()."', '".$this->getWidth()."', '".$this->getHeight()."');".$strp."
				tree.generate(".$this->bigTree.");
			}
		";
		return $str;
	}
	
	function getStructure ($debug = false) {
		$tab = '';
		$tabModel = ($debug) ? '&nbsp;&nbsp;&nbsp;&nbsp;' : '';
		$rt = ($debug) ? '<br />' : '';
		$virgule = '';
		$str = '[';
		$str .= $this->_getJSONChildren($debug, $virgule, $rt, $tab, $tabModel);
		$str .= $rt.']';
		return $str;
	}
	
	function getJSON ($debug = false) {
		$tab = '';
		$tabModel = ($debug) ? '&nbsp;&nbsp;&nbsp;&nbsp;' : '';
		$rt = ($debug) ? '<br />' : '';
		$virgule = '';
		$str = '([';
		$str .= $this->_getJSONChildren($debug, $virgule, $rt, $tab, $tabModel);
		$str .= $rt.'])';
		return $str;
	}
	
	
	
	/**
	 *------------------------------------------------------------------------------
	 *							Fonctions privées
	 *------------------------------------------------------------------------------
	 */
	
	function _getJSONChildren ($debug, $virgule, $rt, $tab, $tabModel) {
		$str = '';
		foreach ($this->branches as $branch) {
			$str .= $branch->getJSON($debug, $virgule, $rt, $tab, $tabModel);
			$virgule = ',';
		}
		return $str;
	}
}

?>